@@ -1,13 +1,16 @@
 import hashlib
 import itertools
+import logging
 from typing import Dict, List, Optional, Set, Tuple, Union
 
 import numpy as np
 import pandas as pd
 from pandas._typing import DtypeObj
 
-from upgini.autofe.all_operands import find_op
-from upgini.autofe.operand import Operand, PandasOperand
+from upgini.autofe.all_operators import find_op
+from upgini.autofe.operator import Operator, PandasOperator
+from upgini.autofe.timeseries.base import TimeSeriesBase
+from upgini.autofe.utils import pydantic_dump_method, pydantic_parse_method
 
 
 class Column:
@@ -16,14 +19,30 @@ class Column:
         self.data = data
         self.calculate_all = calculate_all
 
-    def get_display_name(self, cache: bool = True, shorten: bool = False, **kwargs) -> str:
-        return self.name
-
-    def set_op_params(self, params: Dict[str, str]) -> "Column":
+    def set_op_params(self, params: Dict[str, str], **kwargs) -> "Column":
         return self
 
+    def get_op_params(self, **kwargs):
+        return dict()
+
     def rename_columns(self, mapping: Dict[str, str]) -> "Column":
-        self.name = self._unhash(mapping.get(self.name) or self.name)
+        self.name = mapping.get(self.name) or self.name
+        return self
+
+    def delete_data(self):
+        self.data = None
+
+    def get_column_nodes(self) -> List["Column"]:
+        return [self]
+
+    def get_columns(self, unhash=False, **kwargs):
+        name = self.name
+        return [self._unhash(name) if unhash else name]
+
+    def get_display_name(self, cache: bool = True, shorten: bool = False, **kwargs) -> str:
+        return self.get_columns(**kwargs)[0]
+
+    def reset_display_indices(self) -> "Column":
         return self
 
     def _unhash(self, feature_name: str) -> str:
@@ -35,14 +54,9 @@ class Column:
         else:
             return feature_name[2:last_component_idx]
 
-    def delete_data(self):
-        self.data = None
-
-    def get_column_nodes(self) -> List["Column"]:
-        return [self]
-
-    def get_columns(self, **kwargs) -> List[str]:
-        return [self.name]
+    @property
+    def children(self) -> List[Union["Feature", "Column"]]:
+        return []
 
     def infer_type(self, data: pd.DataFrame) -> DtypeObj:
         return data[self.name].dtype
@@ -67,7 +81,7 @@ class Column:
 class Feature:
     def __init__(
         self,
-        op: Operand,
+        op: Operator,
         children: List[Union[Column, "Feature"]],
         data: Optional[pd.DataFrame] = None,
         display_index: Optional[str] = None,
@@ -81,27 +95,52 @@ class Feature:
         self.cached_display_name = cached_display_name
         self.alias = alias
 
-    def set_op_params(self, params: Optional[Dict[str, str]]) -> "Feature":
-        obj_dict = self.op.dict().copy()
+    def set_op_params(self, params: Optional[Dict[str, str]], **kwargs) -> "Feature":
+        obj_dict = pydantic_dump_method(self.op)().copy()
         obj_dict.update(params or {})
-        self.op = self.op.__class__.parse_obj(obj_dict)
+        self.op = pydantic_parse_method(self.op.__class__)(obj_dict)
         self.op.set_params(params)
 
         for child in self.children:
-            child.set_op_params(params)
+            child_params = {
+                k[len(child.get_display_name(**kwargs)) + 1 :]: v
+                for k, v in params.items()
+                if k.startswith(child.get_display_name(**kwargs))
+            }
+            if not child_params:
+                child_params = params
+            child.set_op_params(child_params, **kwargs)
         return self
 
+    def get_op_params(self, **kwargs) -> Dict[str, str]:
+        return {
+            k: str(v)
+            for k, v in dict(
+                (
+                    (f"{child.get_display_name(**kwargs)}_{k}", v)
+                    for child in self.children
+                    for k, v in child.get_op_params(**kwargs).items()
+                ),
+                **(self.op.get_params() or {}),
+            ).items()
+            if v is not None
+        }
+
     def get_hash(self) -> str:
         return hashlib.sha256(
-            "_".join([self.op.name] + [ch.get_display_name() for ch in self.children]).encode("utf-8")
-        ).hexdigest()[:8]
+            "_".join(
+                [self.op.get_hash_component()]
+                + [ch.op.get_hash_component() for ch in self.children if isinstance(ch, Feature)]
+                + [ch.get_display_name() for ch in self.children]
+            ).encode("utf-8")
+        ).hexdigest()[:10]
 
     def set_alias(self, alias: str) -> "Feature":
         self.alias = alias
         return self
 
     def get_all_operand_names(self) -> Set[str]:
-        return {self.op.name}.union(
+        return {self.op.to_formula()}.union(
             {n for f in self.children if isinstance(f, Feature) for n in f.get_all_operand_names()}
         )
 
@@ -111,6 +150,13 @@ class Feature:
         self.cached_display_name = None
         return self
 
+    def rename_op_params(self, mapping: Dict[str, str]) -> "Feature":
+        self.op.rename_params(mapping)
+        for child in self.children:
+            if isinstance(child, Feature):
+                child.rename_op_params(mapping)
+        return self
+
     def get_column_nodes(self) -> List[Union[Column, "Feature"]]:
         res = []
         for child in self.children:
@@ -128,24 +174,41 @@ class Feature:
 
     def delete_data(self):
         self.data = None
+        self.op.delete_data()
         for child in self.children:
             child.delete_data()
 
-    def get_op_display_name(self) -> str:
-        return self.op.alias or self.op.name.lower()
+    def get_op_display_name(self, use_alias: bool = True) -> str:
+        return (self.op.alias or self.op.to_formula()).lower() if use_alias else self.op.to_formula()
 
-    def get_display_name(self, cache: bool = True, shorten: bool = False, **kwargs) -> str:
+    def get_display_name(self, cache: bool = True, shorten: bool = False, use_op_alias: bool = True, **kwargs) -> str:
         if self.cached_display_name is not None and cache:
             return self.cached_display_name
 
+        should_stack_op = (
+            not isinstance(self.children[-1], Column)
+            if self.op.is_unary or isinstance(self.op, TimeSeriesBase)
+            else False
+        )
+        components = []
+
         if self.alias:
-            components = ["f_autofe", self.alias]
+            components.extend(["f_autofe", self.alias])
+        elif should_stack_op:
+            components.extend(
+                [
+                    self.children[-1].get_display_name(
+                        cache=cache, shorten=shorten, use_op_alias=use_op_alias, **kwargs
+                    ),
+                    self.get_op_display_name(use_alias=use_op_alias),
+                ]
+            )
         elif shorten and not self.op.is_unary:
-            components = ["f_autofe", self.get_op_display_name()]
+            components.extend(["f_autofe", self.get_op_display_name(use_alias=use_op_alias)])
         else:
             components = ["f_" + "_f_".join(self.get_columns(**kwargs))] + [
                 "autofe",
-                self.get_op_display_name(),
+                self.get_op_display_name(use_alias=use_op_alias),
             ]
         components.extend([str(self.display_index)] if self.display_index is not None else [])
         display_name = "_".join(components)
@@ -159,6 +222,13 @@ class Feature:
         self.cached_display_name = None
         return self
 
+    def reset_display_indices(self) -> "Feature":
+        for child in self.children:
+            child.reset_display_indices()
+        self.display_index = None
+        self.cached_display_name = None
+        return self
+
     def infer_type(self, data: pd.DataFrame) -> Union[str, DtypeObj]:
         if self.op.output_type:
             return self.op.output_type
@@ -167,7 +237,7 @@ class Feature:
             return self.children[0].infer_type(data)
 
     def calculate(self, data: pd.DataFrame, is_root=False) -> Union[pd.Series, pd.DataFrame]:
-        if isinstance(self.op, PandasOperand):
+        if isinstance(self.op, PandasOperator):
             if self.op.is_vector:
                 ds = [child.calculate(data) for child in self.children]
                 new_data = self.op.calculate(data=ds)
@@ -209,9 +279,9 @@ class Feature:
         if self.op.name in ["+", "-", "*", "/"]:
             left = self.children[0].to_formula(**kwargs)
             right = self.children[1].to_formula(**kwargs)
-            return f"({left}{self.op.name}{right})"
+            return f"({left}{self.op.to_formula()}{right})"
         else:
-            result = [self.op.name, "("]
+            result = [self.op.to_formula(), "("]
             for i in range(len(self.children)):
                 string_i = self.children[i].to_formula(**kwargs)
                 result.append(string_i)
@@ -224,9 +294,9 @@ class Feature:
         if self.op.name in ["+", "-", "*", "/"]:
             left = self.children[0].to_pretty_formula()
             right = self.children[1].to_pretty_formula()
-            return f"{left} {self.op.name} {right}"
+            return f"{left} {self.op.to_formula()} {right}"
         else:
-            result = [self.op.name, "("]
+            result = [self.op.to_formula(), "("]
             for i in range(len(self.children)):
                 string_i = self.children[i].to_pretty_formula()
                 result.append(string_i)
@@ -237,12 +307,19 @@ class Feature:
 
     @staticmethod
     def from_formula(string: str) -> Union[Column, "Feature"]:
-        if string[-1] != ")":
-            return Column(string)
 
         def is_trivial_char(c: str) -> bool:
             return c not in "()+-*/,"
 
+        if string[-1] != ")":
+            if all(is_trivial_char(c) for c in string):
+                return Column(string)
+            else:
+                raise ValueError(
+                    f"Unsupported column name: {string}. Column names should not have characters: "
+                    "['(', ')', '+', '-', '*', '/', ',']"
+                )
+
         def find_prev(string: str) -> int:
             if string[-1] != ")":
                 return max([(0 if is_trivial_char(c) else i + 1) for i, c in enumerate(string)])
@@ -264,8 +341,11 @@ class Feature:
             return Feature(find_op(string[: p2 - 1]), [Feature.from_formula(string[p2:-1])])
         p1 = find_prev(string[: p2 - 1])
         if string[0] == "(":
+            op = find_op(string[p2 - 1])
+            if op is None:
+                raise ValueError(f"Unsupported operand: {string[p2 - 1]}")
             return Feature(
-                find_op(string[p2 - 1]),
+                op,
                 [Feature.from_formula(string[p1 : p2 - 1]), Feature.from_formula(string[p2:-1])],
             )
         else:
@@ -276,6 +356,8 @@ class Feature:
                     [Feature.from_formula(string[p1 : p2 - 1]), Feature.from_formula(string[p2:-1])],
                 )
             else:
+                if string[p1 - 1] == "(":
+                    raise ValueError(f"Unsupported operand: {string[: p1 - 1]}")
                 base_features = [
                     Feature.from_formula(string[p2:-1]),
                     Feature.from_formula(string[p1 : p2 - 1]),
@@ -288,10 +370,17 @@ class Feature:
                 base_features.reverse()
                 return Feature(op, base_features)
 
+    def set_logger(self, logger: logging.Logger):
+        self.op.set_logger(logger)
+        for child in self.children:
+            if hasattr(child, "set_logger"):
+                child.set_logger(logger)
+        return self
+
 
 class FeatureGroup:
     def __init__(
-        self, op: Operand, main_column: Optional[Union[Column, Feature]], children: List[Union[Column, Feature]]
+        self, op: Operator, main_column: Optional[Union[Column, Feature]], children: List[Union[Column, Feature]]
     ):
         self.op = op
         self.main_column_node = main_column
@@ -312,24 +401,26 @@ class FeatureGroup:
         return names
 
     def calculate(self, data: pd.DataFrame, is_root=False) -> pd.DataFrame:
-        main_column = None if self.main_column_node is None else self.main_column_node.get_columns()[0]
-        if isinstance(self.op, PandasOperand):
-            columns = self.get_columns()
-            lower_order_children = [
+        if isinstance(self.op, PandasOperator):
+            main_column = None if self.main_column_node is None else self.main_column_node.get_display_name()
+            lower_order_children = []
+            if self.main_column_node is not None:
+                lower_order_children.append(self.main_column_node)
+            lower_order_children.extend(
                 ch for f in self.children for ch in f.children if ch.get_display_name() != main_column
-            ]
+            )
             lower_order_names = [ch.get_display_name() for ch in lower_order_children]
-            if any(isinstance(f, Feature) for f in lower_order_children):
-                child_data = pd.concat(
-                    [data[main_column]] + [ch.calculate(data) for ch in lower_order_children],
-                    axis=1,
-                )
-                child_data.columns = [main_column] + lower_order_names
-            else:
-                child_data = data[columns]
+            child_data = pd.concat(
+                [ch.calculate(data) for ch in lower_order_children],
+                axis=1,
+            )
+            child_data.columns = lower_order_names
 
             new_data = self.op.calculate_group(child_data, main_column=main_column)
-            new_data.rename(columns=dict(zip(lower_order_names, self.get_display_names())), inplace=True)
+            new_data.rename(
+                columns=dict(zip((n for n in lower_order_names if n != main_column), self.get_display_names())),
+                inplace=True,
+            )
         else:
             raise NotImplementedError(f"Unrecognized operator {self.op.name}.")
 
@@ -343,7 +434,7 @@ class FeatureGroup:
     def make_groups(candidates: List[Feature]) -> List[Union[Feature, "FeatureGroup"]]:
         grouped_features = []
 
-        def groupby_func(f: Feature) -> Tuple[Operand, Union[Column, Feature]]:
+        def groupby_func(f: Feature) -> Tuple[Operator, Union[Column, Feature]]:
             return (f.op, f.children[0 if not f.op.is_vectorizable else f.op.group_index])
 
         for op_child, features in itertools.groupby(candidates, groupby_func):
