@@ -36,6 +36,8 @@ class FileColumnMeaningType(Enum):
     SCORE = "SCORE"
     TARGET = "TARGET"
     FEATURE = "FEATURE"
+    GENERATED_FEATURE = "GENERATED_FEATURE"
+    DATE_FEATURE = "DATE_FEATURE"
     CUSTOM_KEY = "CUSTOM_KEY"
     COUNTRY = "COUNTRY"
     POSTAL_CODE = "POSTAL_CODE"
@@ -163,7 +165,9 @@ class ModelTaskType(Enum):
         return self in [ModelTaskType.BINARY, ModelTaskType.MULTICLASS]
 
     @staticmethod
-    def parse(task_type: Any) -> "ModelTaskType":
+    def parse(task_type: Any) -> Optional["ModelTaskType"]:
+        if task_type is None:
+            return None
         if isinstance(task_type, ModelTaskType):
             return task_type
         elif isinstance(task_type, str):
@@ -285,6 +289,7 @@ class FeaturesMetadataV2(BaseModel):
     doc_link: Optional[str] = None
     update_frequency: Optional[str] = None
     from_online_api: Optional[bool] = None
+    psi_value: Optional[float] = None
 
 
 class HitRateMetrics(BaseModel):
@@ -326,13 +331,6 @@ class ProviderTaskMetadataV2(BaseModel):
     generated_features: Optional[List[GeneratedFeatureMetadata]] = None
 
 
-class FeaturesFilter(BaseModel):
-    minImportance: Optional[float] = None
-    maxPSI: Optional[float] = None
-    maxCount: Optional[int] = None
-    selectedFeatures: Optional[List[str]] = None
-
-
 class RuntimeParameters(BaseModel):
     properties: Dict[str, Any] = {}
 
@@ -342,11 +340,8 @@ class AutoFEParameters(BaseModel):
 
 
 class SearchCustomization(BaseModel):
-    featuresFilter: Optional[FeaturesFilter] = None
     extractFeatures: Optional[bool] = None
     accurateModel: Optional[bool] = None
-    importanceThreshold: Optional[float] = None
-    maxFeatures: Optional[int] = None
     returnScores: Optional[bool] = None
     runtimeParameters: Optional[RuntimeParameters] = None
     metricsCalculation: Optional[bool] = None
