@@ -1,6 +1,7 @@
 import logging
 import tempfile
 import time
+import uuid
 from functools import lru_cache
 from typing import Dict, List, Optional
 
@@ -97,10 +98,7 @@ class SearchTask:
                     time.sleep(self.POLLING_DELAY_SECONDS)
         except KeyboardInterrupt as e:
             if not check_fit:
-                print(bundle.get("search_stopping"))
-                self.rest_client.stop_search_task_v2(trace_id, search_task_id)
-                self.logger.warning(f"Search {search_task_id} stopped by user")
-                print(bundle.get("search_stopped"))
+                self._stop(trace_id)
             raise e
         print()
 
@@ -118,6 +116,11 @@ class SearchTask:
                 self.logger.error(f"Search failed with errors: {','.join(error_messages)}")
                 raise RuntimeError(bundle.get("all_providers_failed_with_error").format(",".join(error_messages)))
 
+        if check_fit and self.summary.task_type != "INITIAL":
+            raise RuntimeError(
+                bundle.get("search_task_not_initial").format(self.search_task_id, self.summary.initial_search_task_id)
+            )
+
         if self.summary.status in ["COMPLETED", "VALIDATION_COMPLETED"] or (
             check_fit and "VALIDATION" in self.summary.status
         ):
@@ -133,6 +136,14 @@ class SearchTask:
 
         return self
 
+    def _stop(self, trace_id: Optional[str] = None):
+        trace_id = trace_id or uuid.uuid4()
+        search_task_id = self.initial_search_task_id if self.initial_search_task_id is not None else self.search_task_id
+        print(bundle.get("search_stopping"))
+        self.rest_client.stop_search_task_v2(trace_id, search_task_id)
+        self.logger.warning(f"Search {search_task_id} stopped by user")
+        print(bundle.get("search_stopped"))
+
     def get_all_features_metadata_v2(self) -> Optional[List[FeaturesMetadataV2]]:
         if self.provider_metadata_v2 is None:
             return None
@@ -154,7 +165,7 @@ class SearchTask:
 
         return list(zero_hit_search_keys)
 
-    def get_features_for_transform(self) -> Optional[List[str]]:
+    def get_features_for_embeddings(self) -> Optional[List[str]]:
         if self.provider_metadata_v2 is None:
             return None
 
@@ -165,6 +176,24 @@ class SearchTask:
 
         return list(features_for_transform)
 
+    def get_features_for_transform(self) -> List[str]:
+        if self.provider_metadata_v2 is None:
+            return []
+
+        features_for_transform = set()
+        for meta in self.provider_metadata_v2:
+            if meta.features_used_for_embeddings is not None:
+                features_for_transform.update(meta.features_used_for_embeddings)
+            if meta.generated_features:
+                features_for_transform.update(
+                    c.original_name
+                    for f in meta.generated_features
+                    for c in f.base_columns
+                    if c.ads_definition_id is None
+                    and not c.original_name.endswith("_emb")  # embeddings already added
+                )
+        return list(features_for_transform)
+
     def get_shuffle_kfold(self) -> Optional[bool]:
         if self.provider_metadata_v2 is None:
             return None
@@ -294,6 +323,12 @@ class SearchTask:
     def get_file_metadata(self, trace_id: str) -> FileMetadata:
         return self.rest_client.get_search_file_metadata(self.search_task_id, trace_id)
 
+    def update_selected_features(self, trace_id: str, selected_features: list[str]):
+        self.rest_client.update_selected_features(trace_id, self.search_task_id, selected_features)
+
+    def get_selected_features(self, trace_id: str) -> list[str] | None:
+        return self.rest_client.get_selected_features(trace_id, self.search_task_id)
+
 
 @lru_cache
 def _get_all_initial_raw_features_cached(
