@@ -4,11 +4,12 @@ import textwrap
 import urllib.parse
 import uuid
 from datetime import datetime, timezone
-from io import BytesIO
+from io import StringIO
 from typing import Callable, List, Optional
 
 import pandas as pd
 from xhtml2pdf import pisa
+
 from upgini.__about__ import __version__
 
 
@@ -72,7 +73,9 @@ def make_table(df: pd.DataFrame, wrap_long_string=None) -> str:
     )
 
 
-def display_html_dataframe(df: pd.DataFrame, internal_df: pd.DataFrame, header: str):
+def display_html_dataframe(
+    df: pd.DataFrame, internal_df: pd.DataFrame, header: str, display_id: Optional[str] = None, display_handle=None
+):
     if not ipython_available():
         print(header)
         print(internal_df)
@@ -89,9 +92,9 @@ def display_html_dataframe(df: pd.DataFrame, internal_df: pd.DataFrame, header:
     if table_tsv is not None:
         copy_and_share = f"""
             <div style="text-align: right">
-                <button onclick=navigator.clipboard.writeText(decodeURI('{table_tsv}'))>\U0001F4C2 Copy</button>
+                <button onclick=navigator.clipboard.writeText(decodeURI('{table_tsv}'))>\U0001f4c2 Copy</button>
                 <a href='mailto:<Share with...>?subject={email_subject}&body={table_tsv}'>
-                    <button>\U0001F4E8 Share</button>
+                    <button>\U0001f4e8 Share</button>
                 </a>
             </div>"""
     else:
@@ -109,6 +112,7 @@ def display_html_dataframe(df: pd.DataFrame, internal_df: pd.DataFrame, header:
 
             .upgini-df tbody td {{
                 padding: 0.5em;
+                color: black;
             }}
 
             .upgini-df tbody tr:nth-child(odd) {{
@@ -133,7 +137,10 @@ def display_html_dataframe(df: pd.DataFrame, internal_df: pd.DataFrame, header:
             {table_html}
         </div>
         """
-    display(HTML(result_html))
+    if display_handle is not None:
+        return display_handle.update(HTML(result_html))
+    else:
+        return display(HTML(result_html), display_id=display_id)
 
 
 def make_html_report(
@@ -144,20 +151,27 @@ def make_html_report(
     search_id: str,
     email: Optional[str] = None,
     search_keys: Optional[List[str]] = None,
-):
-    # relevant_features_df = relevant_features_df.copy()
-    # relevant_features_df["Feature name"] = relevant_features_df["Feature name"].apply(
-    #     lambda x: "*" + x if x.contains("_autofe_") else x
-    # )
+) -> str:
     relevant_datasources_df = relevant_datasources_df.copy()
     relevant_datasources_df["action"] = (
         f"""<a href="https://upgini.com/request-a-quote?search-id={search_id}">"""
         """<button type="button">Request a quote</button></a>"""
     )
     relevant_datasources_df.rename(columns={"action": "&nbsp;"}, inplace=True)
+
+    try:
+        from importlib.resources import files
+
+        font_path = files("upgini.utils").joinpath("Roboto-Regular.ttf")
+    except Exception:
+        from pkg_resources import resource_filename
+
+        font_path = resource_filename("upgini.utils", "Roboto-Regular.ttf")
+
     return f"""<html>
         <head>
             <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+            <meta charset="UTF-8">
             <style>
                 @page {{
                     size: a4 portrait;
@@ -178,12 +192,14 @@ def make_html_report(
                 }}
 
                 @font-face {{
-                    font-family: "Alice-Regular";
-                    src: url("/fonts/Alice-Regular.ttf") format("truetype");
+                    font-family: "Roboto";
+                    src: url("{font_path}") format("truetype");
                 }}
 
                 body {{
-                    font-family: "Alice-Regular", Arial, sans-serif;
+                    font-family: "Roboto", sans-serif;
+                    font-weight: 400;
+                    font-style: normal;
                 }}
 
                 #header_content {{
@@ -253,17 +269,22 @@ def make_html_report(
                 if search_keys is not None
                 else ""
             }
-            {"<h3>All relevant features. Accuracy after enrichment</h3>" + make_table(metrics_df)
-             if metrics_df is not None
-             else ""
+            {
+                "<h3>All relevant features. Accuracy after enrichment</h3>" + make_table(metrics_df)
+                if metrics_df is not None
+                else ""
+            }
+            {
+                "<h3>Relevant data sources</h3>" + make_table(relevant_datasources_df)
+                if len(relevant_datasources_df) > 0
+                else ""
             }
-            <h3>Relevant data sources</h3>
-            {make_table(relevant_datasources_df)}
             <h3>All relevant features. Listing ({len(relevant_features_df)} items)</h3>
             {make_table(relevant_features_df, wrap_long_string=25)}
-            {"<h3>Description of AutoFE feature names</h3>" + make_table(autofe_descriptions_df, wrap_long_string=25)
-             if autofe_descriptions_df is not None
-             else ""
+            {
+                "<h3>Description of AutoFE feature names</h3>" + make_table(autofe_descriptions_df, wrap_long_string=25)
+                if autofe_descriptions_df is not None and len(autofe_descriptions_df) > 0
+                else ""
             }
             <p>To buy found data sources, please contact: <a href='mailto:sales@upgini.com'>sales@upgini.com</a></p>
             <p>Best regards, </br><b>Upgini Team</b></p>
@@ -279,6 +300,8 @@ def prepare_and_show_report(
     search_id: str,
     email: Optional[str],
     search_keys: Optional[List[str]] = None,
+    display_id: Optional[str] = None,
+    display_handle=None,
 ):
     if not ipython_available():
         return
@@ -288,35 +311,82 @@ def prepare_and_show_report(
     )
 
     if len(relevant_features_df) > 0:
-        show_button_download_pdf(report)
+        return show_button_download_pdf(report, display_id=display_id, display_handle=display_handle)
 
 
-def show_button_download_pdf(source: str, title="\U0001F4CA Download PDF report"):
+def show_button_download_pdf(
+    source: str, title="\U0001f4ca Download PDF report", display_id: Optional[str] = None, display_handle=None
+):
     from IPython.display import HTML, display
 
     file_name = f"upgini-report-{uuid.uuid4()}.pdf"
+
+    # from weasyprint import HTML
+
+    # html = HTML(string=source)
+    # html.write_pdf(file_name)
     with open(file_name, "wb") as output:
-        pisa.CreatePDF(src=BytesIO(source.encode("UTF-8")), dest=output)
+        pisa.CreatePDF(src=StringIO(source), dest=output, encoding="UTF-8")
 
     with open(file_name, "rb") as f:
         b64 = base64.b64encode(f.read())
         payload = b64.decode()
         html = f"""<a download="{file_name}" href="data:application/pdf;base64,{payload}" target="_blank">
         <button>{title}</button></a>"""
-        display(HTML(html))
+        if display_handle is not None:
+            display_handle.update(HTML(html))
+        else:
+            return display(HTML(html), display_id=display_id)
 
 
-def show_request_quote_button():
+def show_request_quote_button(is_registered: bool):
     if not ipython_available():
-        print("https://upgini.com/request-a-quote")
+        if is_registered:
+            print("https://upgini.com/request-a-quote")
+        else:
+            print("https://profile.upgini.com/login")
     else:
-        import ipywidgets as widgets
-        from IPython.display import Javascript, display
+        from IPython.display import HTML, display, Javascript
+        from ipywidgets import Layout, Button
 
-        button = widgets.Button(description="Request a quote", button_style="danger")
+        if is_registered:
+            display(HTML("""
+                <style>
+                    button.custom-button {
+                        border: 1px solid black !important;
+                        background: white !important;
+                        color: black !important;
+                        white-space: nowrap;
+                    }
+                </style>
+            """))
+            description = "Request a quote"
+            tooltip = "Ask a quote"
+            url = "https://upgini.com/request-a-quote"
+        else:
+            display(HTML("""
+                <style>
+                    button.custom-button {
+                        border: 1px solid #d00 !important;
+                        background: #fff !important;
+                        color: #d00 !important;
+                        white-space: nowrap;
+                    }
+                </style>
+            """))
+            description = "Get an API KEY"
+            tooltip = "Register"
+            url = "https://profile.upgini.com/login"
+
+        button = Button(
+            description=description,
+            layout=Layout(width='auto'),
+            tooltip=tooltip
+        )
+        button.add_class("custom-button")
 
         def on_button_clicked(b):
-            display(Javascript('window.open("https://upgini.com/request-a-quote");'))
+            display(Javascript('window.open("' + url + '");'))
 
         button.on_click(on_button_clicked)
 
