@@ -24,7 +24,7 @@ class FeaturesValidator:
         features_for_generate: Optional[List[str]] = None,
         columns_renaming: Optional[Dict[str, str]] = None,
     ) -> Tuple[List[str], List[str]]:
-        # one_hot_encoded_features = []
+        one_hot_encoded_features = []
         empty_or_constant_features = []
         high_cardinality_features = []
         warnings = []
@@ -36,23 +36,17 @@ class FeaturesValidator:
             value_counts = column.value_counts(dropna=False, normalize=True)
             most_frequent_percent = value_counts.iloc[0]
 
-            if most_frequent_percent >= 0.99:
+            if len(value_counts) == 1:
                 empty_or_constant_features.append(f)
+            elif most_frequent_percent >= 0.99:
+                if self.is_one_hot_encoded(column):
+                    one_hot_encoded_features.append(f)
+                else:
+                    empty_or_constant_features.append(f)
 
-            # TODO implement one-hot encoding check
-            # if len(value_counts) == 1:
-            #     empty_or_constant_features.append(f)
-            # elif most_frequent_percent >= 0.99:
-            #     empty_or_constant_features.append(f)
-            #     if set(value_counts.index.to_list()) == {0, 1}:
-            #         one_hot_encoded_features.append(f)
-            #     else:
-            #         empty_or_constant_features.append(f)
-            #     continue
-
-        # if one_hot_encoded_features:
-        #     msg = bundle.get("one_hot_encoded_features").format(one_hot_encoded_features)
-        #     warnings.append(msg)
+        if one_hot_encoded_features:
+            msg = bundle.get("one_hot_encoded_features").format(one_hot_encoded_features)
+            warnings.append(msg)
 
         columns_renaming = columns_renaming or {}
 
@@ -102,3 +96,41 @@ class FeaturesValidator:
     @staticmethod
     def find_constant_features(df: pd.DataFrame) -> List[str]:
         return [i for i in df if df[i].nunique() <= 1]
+
+    @staticmethod
+    def is_one_hot_encoded(series: pd.Series) -> bool:
+        try:
+            # All rows should be the same type
+            if series.apply(lambda x: type(x)).nunique() != 1:
+                return False
+
+            # First, handle string representations of True/False
+            series_copy = series.copy()
+            if series_copy.dtype == "object" or series_copy.dtype == "string":
+                # Convert string representations of boolean values to numeric
+                series_copy = series_copy.astype(str).str.strip().str.lower()
+                series_copy = series_copy.replace({"true": "1", "false": "0"})
+
+            # Column contains only 0 and 1 (as strings or numbers or booleans)
+            series_copy = series_copy.astype(float)
+            if set(series_copy.unique()) != {0.0, 1.0}:
+                return False
+
+            series_copy = series_copy.astype(int)
+
+            # Column doesn't contain any NaN, np.NaN, space, null, etc.
+            if not (series_copy.isin([0, 1])).all():
+                return False
+
+            vc = series_copy.value_counts()
+            # Column should contain both 0 and 1
+            if len(vc) != 2:
+                return False
+
+            # Minority class is 1
+            if vc[1] >= vc[0]:
+                return False
+
+            return True
+        except ValueError:
+            return False
